package jmetal.operators.mutation;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import jmetal.core.Solution;
import jmetal.encodings.solutionType.ArrayRealSolutionType;
import jmetal.encodings.solutionType.RealSolutionType;
import jmetal.util.Configuration;
import jmetal.util.JMException;
import jmetal.util.PseudoRandom;
import jmetal.util.wrapper.XReal;

public class GaussianMutation extends Mutation {

	private Double mutationProbability_ = null;

	private static final List VALID_TYPES = Arrays.asList(
			RealSolutionType.class, ArrayRealSolutionType.class);

	public GaussianMutation(HashMap<String, Object> parameters) {
		super(parameters);
		if (parameters.get("probability") != null)
			mutationProbability_ = (Double) parameters.get("probability");
	} // PolynomialMutation

	
	public void doMutation(double probability, Solution solution) throws JMException {        
	
		XReal x = new XReal(solution) ;		
		for (int var=0; var < solution.numberOfVariables(); var++) {
			if (PseudoRandom.randDouble() <= probability)
			{
				double gauss = 0;
				for (int i = 0; i < 6; i++){
					gauss += PseudoRandom.randDouble();
				}
				gauss /= 6;
				
				double lower = x.getLowerBound(var);
				double upper = x.getUpperBound(var);
				double value = x.getValue(var);
				
				if (value - lower > upper - value){
					lower = 2 * value - upper;
				}
				else {
					upper = 2 * value - lower;
				}
				gauss *= upper - lower;
				
				gauss += lower;
				
				if (gauss < x.getLowerBound(var))
					gauss = x.getLowerBound(var);
				else if (gauss > x.getUpperBound(var))
					gauss = x.getUpperBound(var);

				x.setValue(var, gauss);
			}
		} // for

	} // doMutation
	
	public Object execute(Object object) throws JMException {
		Solution solution = (Solution)object;

		if (!VALID_TYPES.contains(solution.getType().getClass())) {
			Configuration.logger_.severe("GaussianMutation.execute: the solution " +
					"type " + solution.getType() + " is not allowed with this operator");

			Class cls = java.lang.String.class;
			String name = cls.getName(); 
			throw new JMException("Exception in " + name + ".execute()") ;
		} // if 

		doMutation(mutationProbability_, solution);
		return solution;      
	} // execute
}
